
#include <errno.h>
#include <stdio.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>

#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include "i2c-dev.h"
#include <unistd.h>

// various register macros
// read register
#define READ_REGISTER_AUXYCOUT_SYNC(X)		((X & 0x08) == 0x08)
#define READ_REGISTER_TVYCOUT_SYNC(X)		((X & 0x04) == 0x04)
#define READ_REGISTER_FUNCTIONCONTROLINPUT(X)	(X & 0x03)

// write register 0
#define WRITE_REGISTER_VOLUME(X)		(X&0x3F)
#define WRITE_REGISTER_TV_AUDIO_MUTE(X)  ((X&0x01) << 6)
#define WRITE_REGISTER_AUX_AUDIO_MUTE(X) ((X&0x01) << 7)
// write register 1
#define WRITE_REGISTER_TVAV_SRC(X)		(X&0x07)
#define WRITE_REGISTER_AUXAV_SRC(X)	((X&0x07) << 3)
#define WRITE_REGISTER_TVRGB_MUTE(X)	((X&0x01) << 6)
#define WRITE_REGISTER_RGB_SYNC_SRC(X)	((X&0x01) << 7)
// write register 2
#define WRITE_REGISTER_CONTROLVOLTAGE(X) (X&0x03)
#define WRITE_REGISTER_CONTROLPIN(X)	((X&0x03) << 2)
#define WRITE_REGISTER_RGBGAIN(X)		((X&0x03) << 4)
#define WRITE_REGISTER_BLANKOUTPUT(X)	((X&0x03) << 6)
// write register 3
#define WRITE_REGISTER_DO0(X)			(X&0x01)
#define WRITE_REGISTER_DO1(X)			((X&0x01) << 1)
#define WRITE_REGISTER_AUDIOGAIN(X)		((X&0x03) << 2)
#define WRITE_REGISTER_TV_STEREOCTRL(X)		((X&0x03) << 4)
#define WRITE_REGISTER_AUX_STEREOCTRL(X)		((X&0x03) << 6)
// write register 4
#define WRITE_REGISTER_TVVOL_SELECT1(X)	(X&0x01)
#define WRITE_REGISTER_TVVOL_SELECT2(X)	((X&0x01) << 1)
#define WRITE_REGISTER_GAIN6DB(X)	((X&0x01) << 2)
#define WRITE_REGISTER_LINEOUT_SRC(X)	((X&0x01) << 3)

int main(int argc, char *argv[])
{
  int file;
  int adapter_nr = 0;	/* i2c-adpater - see /dev/i2c-X ... */
  int addr = 0x48;	/* The tdk5002c i2c address */
  char filename[20];
  __u8 devstate;
  __u8 writereg[5];
  __u16 writereg2;
  __u32 parms[20];
  int i;
  
  sprintf(filename,"/dev/i2c-%d",adapter_nr);
  if ((file = open(filename,O_RDWR)) < 0) {
    // do error handling
    fprintf (stderr,"Could not open i2c-adapter: %s\n",filename);
    exit(1);
  }

  if (ioctl(file,I2C_SLAVE,addr) < 0) {
    // do error handling
    fprintf (stderr,"No device with address %X present at adapter %s\n",addr,filename);
    exit(1);
  }
 
  devstate =i2c_smbus_read_byte(file);

  if ((devstate & 0x10) != 0x10)
  {
    // no tdk5002c!
    fprintf (stderr,"Device %X at adapter %s is NOT a TDK5002c\n",addr,filename);
    exit(1);
  }
  
  fprintf (stdout, "TDK5002c read register settings:\n");
  fprintf (stdout,"AUX ycout: %s\n",READ_REGISTER_AUXYCOUT_SYNC(devstate)?"sync pulse present":"no sync pulse");
  fprintf (stdout,"TV ycout: %s\n",READ_REGISTER_TVYCOUT_SYNC(devstate)?"sync pulse present":"no sync pulse");
  
  if (READ_REGISTER_FUNCTIONCONTROLINPUT(devstate) == 0x00)
    fprintf (stdout,"TV_Fnc or AUX_Fnc pin level =Level 0 .~0V\n");
  else if (READ_REGISTER_FUNCTIONCONTROLINPUT(devstate) == 0x01)
    fprintf (stdout,"TV_Fnc or AUX_Fnc pin level =Level 1A ~6.0V\n");
  else if (READ_REGISTER_FUNCTIONCONTROLINPUT(devstate) == 0x02)
    fprintf (stdout,"TV_Fnc or AUX_Fnc pin level = Level 1B ~12V\n");
  else
    fprintf (stdout,"TV_Fnc or AUX_Fnc pin level = _UNKNOWN_ Level - Whats this?\n");
  
  if (argc != 21)
  {
    fprintf(stdout,"Not enough parameters given - will exit\n");
    exit(0);
  }
  
  for (i=0; i<20;i++)
  {
    if (sscanf(argv[i+1],"%X",&parms[i]) != 1)
    {
      fprintf(stderr,"Parameter %d is not in hex-format\n",(i+1));
      exit(1);
    }
  }

  // build write register content
  writereg[0] = 0x00 |
	WRITE_REGISTER_VOLUME(parms[0]) |
	WRITE_REGISTER_TV_AUDIO_MUTE(parms[1]) |
	WRITE_REGISTER_AUX_AUDIO_MUTE(parms[2]);
  writereg[1] = 0x00 |
	WRITE_REGISTER_TVAV_SRC(parms[3]) |
	WRITE_REGISTER_AUXAV_SRC(parms[4]) |
	WRITE_REGISTER_TVRGB_MUTE(parms[5]) |
	WRITE_REGISTER_RGB_SYNC_SRC(parms[6]);
  writereg[2] = 0x00 |
	WRITE_REGISTER_CONTROLVOLTAGE(parms[7]) |
	WRITE_REGISTER_CONTROLPIN(parms[8]) |
	WRITE_REGISTER_RGBGAIN(parms[9]) |
	WRITE_REGISTER_BLANKOUTPUT(parms[10]);
  writereg[3] = 0x00 |
	WRITE_REGISTER_DO0(parms[11]) |
	WRITE_REGISTER_DO1(parms[12]) |
	WRITE_REGISTER_AUDIOGAIN(parms[13]) |
	WRITE_REGISTER_TV_STEREOCTRL(parms[14]) |
	WRITE_REGISTER_AUX_STEREOCTRL(parms[15]);
  writereg[4] = 0x00 |
	WRITE_REGISTER_TVVOL_SELECT1(parms[16]) |
	WRITE_REGISTER_TVVOL_SELECT2(parms[17]) |
	WRITE_REGISTER_GAIN6DB(parms[18]) |
	WRITE_REGISTER_LINEOUT_SRC(parms[19]);

  fprintf(stdout,"About to write (0-4): %X %X %X %X %X\n",writereg[0],writereg[1],writereg[2],writereg[3],writereg[4]);
  	
  // shuffle write-register for our superduper workaround ....
  writereg2 = 0x0000 | writereg[1] | (writereg[2] << 8);
  
  if (i2c_smbus_write_block_data(file, writereg[0], 3, &writereg[2]) != 0)
    fprintf(stderr,"ERROR doing blockwrite\n");
  if (i2c_smbus_write_word_data(file,writereg[0],writereg2) != 0)
    fprintf(stderr,"ERROR doing wordwrite\n");

  close(file);
  
  fprintf (stdout,"All fine - register succ. written!\n");
  
  exit(0);
}

